Package: pkgdepends
Title: Package Dependency Resolution and Downloads
Version: 0.9.1.9000
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Find recursive dependencies of 'R' packages from various
    sources. Solve the dependencies to obtain a consistent set of packages
    to install. Download packages, and install them. It supports packages
    on 'CRAN', 'Bioconductor' and other 'CRAN-like' repositories,
    'GitHub', package 'URLs', and local package trees and files. It caches
    metadata and package files via the 'pkgcache' package, and performs
    all 'HTTP' requests, downloads, builds and installations in parallel.
    'pkgdepends' is the workhorse of the 'pak' package.
License: MIT + file LICENSE
URL: https://r-lib.github.io/pkgdepends/,
        https://github.com/r-lib/pkgdepends
BugReports: https://github.com/r-lib/pkgdepends/issues
Depends: R (>= 3.5)
Imports: callr (>= 3.3.1), cli (>= 3.6.0), curl, desc (>= 1.4.3),
        filelock (>= 1.0.2), jsonlite, lpSolve, pkgbuild (>= 1.0.2),
        pkgcache (>= 2.2.0), processx (>= 3.4.2), ps, R6, stats, utils,
        zip (>= 2.3.0)
Suggests: asciicast (>= 2.2.0.9000), codetools, covr, debugme, fansi,
        fs, gh, gitcreds, glue, htmlwidgets, mockery, pak, pingr (>=
        2.0.0), rmarkdown, rstudioapi, spelling, svglite, testthat (>=
        3.2.0), tibble, webfakes (>= 1.1.5.9000), withr (>= 2.1.1)
Config/Needs/builder: gh, pkgsearch, withr (>= 2.1.1)
Config/Needs/coverage: r-lib/asciicast, covr
Config/Needs/website: r-lib/asciicast, pkgdown (>= 2.0.2),
        tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-05-05
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-21 13:41:27 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-07 08:15:03 UTC; unix
Archs: pkgdepends.so.dSYM
