Package: pak
Title: Another Approach to Package Installation
Version: 0.10.0
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Jim", "Hester", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49")),
    person("Winston", "Chang", role = "ctb",
           comment = "R6, callr, processx"),
    person("Ascent Digital Services", role = c("cph", "fnd"),
           comment = "callr, processx"),
    person("Hadley", "Wickham", role = c("ctb", "cph"),
           comment = "cli, curl, pkgbuild, yaml"),
    person("Jeroen", "Ooms", role = "ctb",
           comment = "curl, jsonlite"),
    person("Maëlle", "Salmon", role = "ctb",
           comment = "desc, pkgsearch"),
    person("Duncan", "Temple Lang", role = "ctb",
           comment = "jsonlite"),
    person("Lloyd", "Hilaiel", role = "cph",
           comment = "jsonlite"),
    person("Alec", "Wong", role = "ctb",
           comment = "keyring"),
    person("Michel Berkelaar and lpSolve authors", role = "ctb",
           comment = "lpSolve"),
    person("R Consortium", role = "fnd",
           comment = "pkgsearch"),
    person("Jay", "Loden", role = "ctb",
           comment = "ps"),
    person("Dave", "Daeschler", role = "ctb",
           comment = "ps"),
    person("Giampaolo", "Rodola", role = "ctb",
           comment = "ps"),
    person("Shawn", "Garbett", role = "ctb",
           comment = "yaml"),
    person("Jeremy", "Stephens", role = "ctb",
           comment = "yaml"),
    person("Kirill", "Simonov", role = "ctb",
           comment = "yaml"),
    person("Yihui", "Xie", role = "ctb",
           comment = "yaml"),
    person("Zhuoer", "Dong", role = "ctb",
           comment = "yaml"),
    person("Jeffrey", "Horner", role = "ctb",
           comment = "yaml"),
    person("Will", "Beasley", role = "ctb",
           comment = "yaml"),
    person("Brendan", "O'Connor", role = "ctb",
           comment = "yaml"),
    person("Gregory", "Warnes", role = "ctb",
           comment = "yaml"),
    person("Michael", "Quinn", role = "ctb",
           comment = "yaml"),
    person("Zhian", "Kamvar", role = "ctb",
           comment = "yaml"),
    person("Charlie", "Gao", role = "ctb",
           comment = "yaml"),
    person("Kuba", "Podgórski", role = "ctb",
           comment = "zip"),
    person("Rich", "Geldreich", role = "ctb",
           comment = "zip")
  )
Description: The goal of 'pak' is to make package installation faster and
    more reliable. In particular, it performs all HTTP operations in
    parallel, so metadata resolution and package downloads are fast.
    Metadata and package files are cached on the local disk as well. 'pak'
    has a dependency solver, so it finds version conflicts before
    performing the installation. This version of 'pak' supports CRAN,
    'Bioconductor' and 'GitHub' packages as well.
License: GPL-3
URL: https://pak.r-lib.org/, https://github.com/r-lib/pak
BugReports: https://github.com/r-lib/pak/issues
Depends: R (>= 3.5)
Imports: tools, utils
Suggests: callr (>= 3.7.0), cli (>= 3.2.0), covr, curl (>= 4.3.2), desc
        (>= 1.4.1), filelock (>= 1.0.2), gitcreds, glue (>= 1.6.2),
        jsonlite (>= 1.8.0), keyring (>= 1.4.0), pingr, pkgbuild (>=
        1.4.2), pkgcache (>= 2.2.4), pkgdepends (>= 0.9.0), pkgload,
        pkgsearch (>= 3.1.0), processx (>= 3.8.1), ps (>= 1.6.0),
        rstudioapi, testthat (>= 3.2.0), webfakes, withr, yaml
Biarch: true
ByteCompile: true
Config/build/extra-sources: configure*
Config/needs/dependencies: callr, cli, curl, desc, filelock, jsonlite,
        keyring, lpSolve, pkgbuild, r-lib/pkgcache@feature/ppm-sso,
        r-lib/pkgdepends, pkgsearch, processx, ps, r-lib/tsitter,
        gaborcsardi/tstoml, yaml
Config/Needs/website: r-lib/asciicast, rmarkdown, roxygen2,
        tidyverse/tidytemplate
Config/Needs/deploy: cli (>= 3.2.0), curl, desc, gitcreds, glue@1.6.2,
        gaborcsardi/jsonlite, processx, R6@2.5.1
Config/Needs/check: r-lib/pkgcache@feature/ppm-sso
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-05-13
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-06 16:52:32 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Jim Hester [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  Winston Chang [ctb] (R6, callr, processx),
  Ascent Digital Services [cph, fnd] (callr, processx),
  Hadley Wickham [ctb, cph] (cli, curl, pkgbuild, yaml),
  Jeroen Ooms [ctb] (curl, jsonlite),
  Maëlle Salmon [ctb] (desc, pkgsearch),
  Duncan Temple Lang [ctb] (jsonlite),
  Lloyd Hilaiel [cph] (jsonlite),
  Alec Wong [ctb] (keyring),
  Michel Berkelaar and lpSolve authors [ctb] (lpSolve),
  R Consortium [fnd] (pkgsearch),
  Jay Loden [ctb] (ps),
  Dave Daeschler [ctb] (ps),
  Giampaolo Rodola [ctb] (ps),
  Shawn Garbett [ctb] (yaml),
  Jeremy Stephens [ctb] (yaml),
  Kirill Simonov [ctb] (yaml),
  Yihui Xie [ctb] (yaml),
  Zhuoer Dong [ctb] (yaml),
  Jeffrey Horner [ctb] (yaml),
  Will Beasley [ctb] (yaml),
  Brendan O'Connor [ctb] (yaml),
  Gregory Warnes [ctb] (yaml),
  Michael Quinn [ctb] (yaml),
  Zhian Kamvar [ctb] (yaml),
  Charlie Gao [ctb] (yaml),
  Kuba Podgórski [ctb] (zip),
  Rich Geldreich [ctb] (zip)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-07 06:30:11 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-07 08:12:17 UTC; unix
