Package: filelock
Title: Portable File Locking
Version: 1.0.3
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Place an exclusive or shared lock on a file. It uses
    'LockFile' on Windows and 'fcntl' locks on Unix-like systems.
License: MIT + file LICENSE
URL: https://r-lib.github.io/filelock/,
        https://github.com/r-lib/filelock
BugReports: https://github.com/r-lib/filelock/issues
Depends: R (>= 3.4)
Suggests: callr (>= 2.0.0), covr, testthat (>= 3.0.0)
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2023-12-11 01:12:05 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2023-12-11 04:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-07 08:12:49 UTC; unix
Archs: filelock.so.dSYM
