Package: tstoml
Title: Edit TOML Files
Version: 0.0.0.9000
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
        comment = c(ROR = "03wc8by49")),
    person(family = "tree-sitter-toml authors", role = "ctb")
  )
Description: Extract and manipulate parts of TOML files without touching
  the formatting and comments in other parts.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Depends: R (>= 3.5.0)
Imports: tsitter
Suggests: asciicast, hms, pillar, testthat (>= 3.0.0), withr
Remotes: r-lib/tsitter
Config/Needs/website: r-lib/asciicast, tidyverse/tidytemplate
Config/testthat/edition: 3
URL: https://github.com/gaborcsardi/tstoml,
        https://gaborcsardi.github.io/tstoml/
BugReports: https://github.com/gaborcsardi/tstoml/issues
Biarch: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-01 13:38:06 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  tree-sitter-toml authors [ctb]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-07 08:14:07 UTC; unix
Archs: tstoml.so.dSYM
