Package: zip
Title: Cross-Platform 'zip' Compression
Version: 2.3.3
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Kuba", "Podgórski", role = "ctb"),
    person("Rich", "Geldreich", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Cross-Platform 'zip' Compression Library. A replacement for
    the 'zip' function, that does not require any additional external
    tools on any platform.
License: MIT + file LICENSE
URL: https://github.com/r-lib/zip, https://r-lib.github.io/zip/
BugReports: https://github.com/r-lib/zip/issues
Suggests: covr, pillar, processx, R6, testthat, withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-05-07
Encoding: UTF-8
RoxygenNote: 7.3.2.9000
NeedsCompilation: yes
Packaged: 2025-05-13 13:31:44 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Kuba Podgórski [ctb],
  Rich Geldreich [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2025-05-13 14:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-27 21:18:12 UTC; unix
