# Generated by roxygen2: do not edit by hand

export(deflate)
export(inflate)
export(unzip)
export(unzip_process)
export(zip)
export(zip_append)
export(zip_list)
export(zip_process)
export(zipr)
export(zipr_append)
useDynLib(zip, .registration = TRUE, .fixes = "c_")
