# Generated by roxygen2: do not edit by hand

S3method(ts_tree_ast,ts_tree_toml)
S3method(ts_tree_delete,ts_tree_toml)
S3method(ts_tree_format,ts_tree_toml)
S3method(ts_tree_insert,ts_tree_toml)
S3method(ts_tree_mark_selection1,ts_tree_toml)
S3method(ts_tree_new,ts_language_toml)
S3method(ts_tree_query,ts_tree_toml)
S3method(ts_tree_select1,ts_tree_toml.ts_tree_selector_default)
S3method(ts_tree_unserialize,ts_tree_toml)
S3method(ts_tree_update,ts_tree_toml)
S3method(ts_tree_write,ts_tree_toml)
export(toml_example_text)
export(ts_format_toml)
export(ts_language_toml)
export(ts_parse_toml)
export(ts_read_toml)
export(ts_serialize_toml)
export(ts_toml_array)
export(ts_toml_array_of_tables)
export(ts_toml_inline_table)
export(ts_toml_table)
export(ts_tree_select.ts_tree_toml)
export(ts_unserialize_toml)
import(tsitter)
useDynLib(tstoml, .registration = TRUE, .fixes = "c_")
