
#' Call R from R
#'
#' @description
#' It is sometimes useful to perform a computation in a separate
#' R process, without affecting the current R process at all.  This
#' packages does exactly that.
#'
#' # callr
#'
#' ```{r man-readme, child = "README.Rmd"}
#' ```
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL
