% Generated by roxygen2: do not edit by hand % Please edit documentation in R/library.R, R/package.R \name{lib_status} \alias{lib_status} \alias{pkg_list} \title{Status of packages in a library} \usage{ lib_status(lib = .libPaths()[1]) pkg_list(lib = .libPaths()[1]) } \arguments{ \item{lib}{Path to library.} } \value{ Data frame the contains data about the packages installed in the library. \eval{include_docs("pkgdepends", "docs/lib-status-return.rds")} } \description{ Status of packages in a library } \section{Examples}{ \if{html}{\out{
    }}\preformatted{lib_status(.Library)
    }\if{html}{\out{
}}\if{html}{\out{

    #> # A data frame: 31 × 31
    #>    library   package version prior…¹ title license sugge…² built depends
    #>    <chr>     <chr>   <chr>   <chr>   <chr> <chr>   <chr>   <chr> <chr>
    #>  1 /Library… base    4.2.2   base    "The… Part o… methods R 4.… NA
    #>  2 /Library… boot    1.3-28  recomm… "Boo… Unlimi… MASS, … R 4.… R (>= …
    #>  3 /Library… class   7.3-20  recomm… "Fun… GPL-2 … NA      R 4.… R (>= …
    #>  4 /Library… cluster 2.1.4   recomm… "\\"F… GPL (>… MASS, … R 4.… R (>= …
    #>  5 /Library… codeto… 0.2-18  recomm… "Cod… GPL     NA      R 4.… R (>= …
    #>  6 /Library… compil… 4.2.2   base    "The… Part o… NA      R 4.… NA
    #>  7 /Library… datase… 4.2.2   base    "The… Part o… NA      R 4.… NA
    #>  8 /Library… filelo… 1.0.2   NA      "Por… MIT + … callr … R 4.… NA
    #>  9 /Library… foreign 0.8-83  recomm… "Rea… GPL (>… NA      R 4.… R (>= …
    #> 10 /Library… graphi… 4.2.2   base    "The… Part o… NA      R 4.… NA
    #> # … with 21 more rows, 22 more variables: needscompilation <lgl>,
    #> #   repository <chr>, imports <chr>, remotetype <chr>,
    #> #   remotepkgref <chr>, remoteref <chr>, remoterepos <chr>,
    #> #   remotepkgplatform <chr>, remotesha <chr>, enhances <chr>,
    #> #   linkingto <chr>, md5sum <chr>, platform <chr>, biocviews <chr>,
    #> #   sysreqs <chr>, ref <chr>, type <chr>, status <chr>, rversion <chr>,
    #> #   sources <list>, repotype <chr>, deps <list>, and abbreviated …

}} } \seealso{ Other package functions: \code{\link{pak}()}, \code{\link{pkg_deps}()}, \code{\link{pkg_deps_tree}()}, \code{\link{pkg_download}()}, \code{\link{pkg_install}()}, \code{\link{pkg_remove}()}, \code{\link{pkg_status}()}, \code{\link{pkg_sysreqs}()} } \concept{library functions} \concept{package functions}