% Generated by roxygen2: do not edit by hand % Please edit documentation in R/package.R \name{pkg_status} \alias{pkg_status} \title{Display installed locations of a package} \usage{ pkg_status(pkg, lib = .libPaths()) } \arguments{ \item{pkg}{Name of one or more installed packages to display status for.} \item{lib}{One or more library paths to lookup packages status in. By default all libraries are used.} } \value{ Data frame with data about installations of \code{pkg}. \eval{include_docs("pkgdepends", "docs/lib-status-return.rds")} } \description{ Display installed locations of a package } \section{Examples}{ \if{html}{\out{
    }}\preformatted{pkg_status("MASS")
    }\if{html}{\out{
}}\if{html}{\out{

    #> # A data frame: 2 × 39
    #>   library    package title version depends repos…¹ license needs…² built
    #> * <chr>      <chr>   <chr> <chr>   <chr>   <chr>   <chr>   <lgl>   <chr>
    #> 1 /Users/ga… MASS    Supp… 7.3-58… R (>= … CRAN    GPL-2 … TRUE    R 4.…
    #> 2 /Library/… MASS    Supp… 7.3-58… R (>= … CRAN    GPL-2 … TRUE    R 4.…
    #> # … with 30 more variables: remotetype <chr>, remotepkgref <chr>,
    #> #   remoteref <chr>, remoterepos <chr>, remotepkgplatform <chr>,
    #> #   remotesha <chr>, imports <chr>, suggests <chr>, linkingto <chr>,
    #> #   remotes <chr>, remotehost <chr>, remoterepo <chr>,
    #> #   remoteusername <chr>, enhances <chr>, biocviews <chr>,
    #> #   remoteurl <chr>, remotesubdir <chr>, priority <chr>,
    #> #   remoteetag <chr>, remotepackaged <chr>, md5sum <chr>, …

}} } \seealso{ Other package functions: \code{\link{lib_status}()}, \code{\link{pak}()}, \code{\link{pkg_deps}()}, \code{\link{pkg_deps_tree}()}, \code{\link{pkg_download}()}, \code{\link{pkg_install}()}, \code{\link{pkg_remove}()}, \code{\link{pkg_sysreqs}()} } \concept{package functions}