% Generated by roxygen2: do not edit by hand % Please edit documentation in R/name-check.R \name{pkg_name_check} \alias{pkg_name_check} \title{Check if an R package name is available} \usage{ pkg_name_check(name, dictionaries = NULL) } \arguments{ \item{name}{Package name candidate.} \item{dictionaries}{Character vector, the dictionaries to query. Available dictionaries: * \code{wikipedia} * \code{wiktionary}, * \code{sentiment} (\url{https://github.com/fnielsen/afinn}), * \code{urban} (Urban Dictionary). If \code{NULL} (by default), the Urban Dictionary is omitted, as it is often offensive.} } \value{ \code{pkg_name_check} object with a custom print method. } \description{ Additionally, look up the candidate name in a number of dictionaries, to make sure that it does not have a negative meaning. } \details{ \subsection{Valid package name check}{ Check the validity of \code{name} as a package name. See 'Writing R Extensions' for the allowed package names. Also checked against a list of names that are known to cause problems. } \subsection{CRAN checks}{ Check \code{name} against the names of all past and current packages on CRAN, including base and recommended packages. } \subsection{Bioconductor checks}{ Check \code{name} against all past and current Bioconductor packages. } \subsection{Profanity check}{ Check \code{name} with \url{https://www.purgomalum.com/service/containsprofanity} to make sure it is not a profanity. } \subsection{Dictionaries}{ See the \code{dictionaries} argument. } } \section{Examples}{ \if{html}{\out{
    }}\preformatted{pkg_name_check("sicily")
    }\if{html}{\out{
}}\if{html}{\out{

    #> ╔══════════════════════════════════════════════════════════════════════╗
    #> ║                            –*– sicily –*–                            ║
    #> ╚══════════════════════════════════════════════════════════════════════╝
    #> ┌──────────────────────────────────────────────────────────────────────┐
    #> │ v  valid name          v  CRAN               v  Bioconductor         │
    #> │ v  not a profanity                                                   │
    #> └──────────────────────────────────────────────────────────────────────┘
    #> ┌ Wikipedia ───────────────────────────────────────────────────────────┐
    #> │ Sicily Sicily (Italian: Sicilia [siˈtʃiːlja], Sicilian               │
    #> │ pronunciation: [sɪˈʃiːlja]) is the largest island in the             │
    #> │ Mediterranean Sea and one of the 20 regions of Italy. The Strait of  │
    #> │ Messina divides it from the region of Calabria in Southern Italy.    │
    #> │ It is one of the five Italian autonomous regions and is officially   │
    #> │ referred to as Regione Siciliana. The region has 5 million           │
    #> │ …                                                                    │
    #> └──────────────────────────────── https://en.wikipedia.org/wiki/Sicily ┘
    #> ┌ Wiktionary ──────────────────────────────────────────────────────────┐
    #> │ sicily No English definition found                                   │
    #> └────────────────────────────────────────────────────────────────────  ┘
    #> ┌──────────────────────────────────────────────────────────────────────┐
    #> │ Sentiment: 😐 (0)                                                    │
    #> └──────────────────────────────────────────────────────────────────────┘

}} }